@@ -1,4 +1,5 @@
 from .conv2plus1d import Conv2plus1d
 from .conv_audio import ConvAudio
+from .tam import TAM
 
-__all__ = ['Conv2plus1d', 'ConvAudio']
+__all__ = ['Conv2plus1d', 'ConvAudio', 'TAM']
