@@ -1,12 +1,16 @@
+from .audio_tsn_head import AudioTSNHead
 from .base import BaseHead
+from .bbox_head import BBoxHeadAVA
 from .i3d_head import I3DHead
+from .roi_head import AVARoIHead
 from .slowfast_head import SlowFastHead
 from .ssn_head import SSNHead
 from .tpn_head import TPNHead
 from .tsm_head import TSMHead
 from .tsn_head import TSNHead
+from .x3d_head import X3DHead
 
 __all__ = [
     'TSNHead', 'I3DHead', 'BaseHead', 'TSMHead', 'SlowFastHead', 'SSNHead',
-    'TPNHead'
+    'TPNHead', 'AudioTSNHead', 'X3DHead', 'BBoxHeadAVA', 'AVARoIHead'
 ]
